@@ -99,12 +99,12 @@ public:
   static void setPenColor(WPen& pen, const WDataSeries& series,
 		   int xRow, int xColumn,
 		   int yRow, int yColumn,
-		   int colorRole);
+		   ItemDataRole colorRole);
 
   static void setBrushColor(WBrush& brush, const WDataSeries& series,
 		     int xRow, int xColumn,
 		     int yRow, int yColumn,
-		     int colorRole);
+		     ItemDataRole colorRole);
 
 private:
   int currentXSegment_, currentYSegment_;
@@ -130,6 +130,13 @@ public:
    */
   CurveLabel(const WDataSeries &series, const WPointF &point, const WT_USTRING &label);
 
+  /*! \brief Create a new curve label.
+   *
+   * Create a new curve label for given series,
+   * at the given x, y coordinates and the given text.
+   */
+  CurveLabel(const WDataSeries &series, const cpp17::any &x, const cpp17::any &y, const WT_USTRING &label);
+
   /*! \brief Set the series this curve label is associated with.
    */
   void setSeries(const WDataSeries &series);
@@ -144,11 +151,31 @@ public:
    */
   void setPoint(const WPointF &point);
 
+  /*! \brief Set the point in model coordinates this label is associated with.
+   */
+  void setPoint(const cpp17::any &x, const cpp17::any &y);
+
   /*! \brief Get the point in model coordinates this label is associated with.
+   *
+   * \note This uses asNumber(), which may not be the same conversion that
+   *       WCartesianChart::mapToDevice() uses, depending on the scale of the
+   *       axis. x() and y() will perform no conversion, so they may be safer to use.
    *
    * \sa setPoint()
    */
-  const WPointF &point() const { return point_; }
+  const WPointF point() const;
+
+  /*! \brief Get the x position for this label.
+   *
+   * \sa setPoint()
+   */
+  const cpp17::any &x() const { return x_; }
+
+  /*! \brief Get the y position for this label.
+   *
+   * \sa setPoint()
+   */
+  const cpp17::any &y() const { return y_; }
 
   /*! \brief Set the label that should be drawn in the box.
    */
@@ -243,7 +270,7 @@ public:
 
 private:
   const WDataSeries *series_;
-  WPointF point_;
+  cpp17::any x_, y_;
   WT_USTRING label_;
   WPointF offset_;
   int width_;
@@ -507,7 +534,7 @@ public:
    *
    * \sa series(), addSeries(WDataSeries *), removeSeries(WDataSeries *)
    */
-  void setSeries(std::vector<std::unique_ptr<WDataSeries>> series);
+  void setSeries(std::vector<std::unique_ptr<WDataSeries> > series);
 
   /*! \brief Returns a data series corresponding to a data column.
    *
@@ -529,7 +556,7 @@ public:
    *
    * \sa setSeries(const std::vector<WDataSeries>&)
    */
-  const std::vector<std::unique_ptr<WDataSeries>>& series() const { return series_; }
+  const std::vector<std::unique_ptr<WDataSeries> >& series() const { return series_; }
 
   /*! \brief Returns a chart axis.
    *
@@ -686,7 +713,7 @@ public:
    * area. When \p location is \link Wt::Chart::LegendLocation::Inside
    * Chart::LegendLocation::Inside\endlink, the legend will be rendered inside
    * the chart. When \p location is \link Wt::Chart::LegendLocation::Outside
-   * Chart::Legendoutside\endlink, the legend is rendered outside the
+   * Chart::LegendLocation::Outside\endlink, the legend is rendered outside the
    * chart, in the chart padding area.
    *
    * The provided \p side can either be Wt::Side::Left, Wt::Side::Right, Wt::Side::Top,
@@ -696,11 +723,14 @@ public:
    * The \p alignment specifies how the legend is aligned. This can be
    * a horizontal alignment flag (Wt::AlignmentFlag::Left, Wt::AlignmentFlag::Center, or
    * Wt::AlignmentFlag::Right), when the \p side is Side::Bottom or Side::Top, or a vertical
-   * alignment flag (Wt::AlignmentFlag::Top, Wt::AlignmentFlag::Middle, or Wt::AlignmentFlag::Bottom)
-   * when the \p side is Side::Left or Side::Right.
+   * alignment flag (\link Wt::AlignmentFlag Wt::AlignmentFlag::Top\endlink,
+   * \link Wt::AlignmentFlag Wt::AlignmentFlag::Middle\endlink, or
+   * \link Wt::AlignmentFlag Wt::AlignmentFlag::Bottom\endlink)
+   * when the \p side is \link Wt::Side Side::Left\endlink or \link Wt::Side Side::Right\endlink.
    *
    * The default location is \link Wt::Chart::LegendLocation::Outside
-   * Chart::LegendLocation::Outside\endlink, Wt::Side::Right and Wt::AlignmentFlag::Middle.
+   * Chart::LegendLocation::Outside\endlink, \link Wt::Side Wt::Side::Right\endlink and
+   * \link Wt::AlignmentFlag Wt::AlignmentFlag::Middle\endlink.
    *
    * To have more control over the legend, you could reimplement the
    * renderLegendItem() method to customize how one item in the legend
@@ -1109,9 +1139,8 @@ public:
    *	     They may be clipped off outside of the chart area, and when
    *         zooming, the axis ticks will change size.
    *       - WAxis::setBreak() is incompatible with interactive mode
-   *
-   * @{
    */
+  //!@{
 
   /*! \brief Returns whether this chart is interactive.
    *
@@ -1422,8 +1451,7 @@ public:
    */
   const WBrush& loadingBackground() const { return loadingBackground_; }
 
-  /*! @}
-   */
+  //!@}
 
   void iterateSeries(SeriesIterator *iterator,
 		     WPainter *painter, bool reverseStacked = false, bool extremesOnly = false) const;
@@ -1486,7 +1514,7 @@ private:
   Orientation orientation_;
   int XSeriesColumn_;
   ChartType type_;
-  std::vector<std::unique_ptr<WDataSeries>> series_;
+  std::vector<std::unique_ptr<WDataSeries> > series_;
   AxisStruct xAxis_;
   std::vector<AxisStruct> yAxes_;
   double barMargin_;
@@ -1526,7 +1554,7 @@ private:
   mutable TransformMap curveTransforms_;
   std::vector<WJavaScriptHandle<WTransform> > freeTransforms_;
 
-  std::vector<WJavaScriptHandle<WPen>> freePens_;
+  std::vector<WJavaScriptHandle<WPen> > freePens_;
 
   std::vector<CurveLabel> curveLabels_;
 
@@ -1564,7 +1592,7 @@ protected:
 
   /** @name Rendering logic
    */
-  //@{
+  //!@{
   /*! \brief Paints the widget.
    *
    * This calls render() to paint on the paint device.
@@ -1749,7 +1777,7 @@ protected:
    * This method can be overridden to draw extra content onto
    * the chart.
    *
-   * Chart coordinates can be mapped to device coordinates with
+   * %Chart coordinates can be mapped to device coordinates with
    * mapToDeviceWithoutTransform(). If these need to move and scale
    * along with the zoom range, those points can be transformed with
    * zoomRangeTransform().
@@ -1764,7 +1792,7 @@ protected:
    */
   int calcNumBarGroups();
 
-  //@}
+  //!@}
 
   virtual void render(WFlags<RenderFlag> flags) override ;
   virtual void setFormData(const FormData& formData) override ;
@@ -1822,6 +1850,9 @@ private:
   WRectF insideChartArea() const;
   int calcAxisSize(const WAxis &axis, WPaintDevice *device) const;
   void defineJavaScript();
+  void drawMarker(const WDataSeries &series,
+                  MarkerType marker,
+                  WPainterPath &result) const;
 
   bool axisSliderWidgetForSeries(WDataSeries *series) const;
 
