@@ -107,6 +107,8 @@ public:
    */
   WBoxLayout(LayoutDirection dir);
 
+  virtual ~WBoxLayout() override;
+
   virtual void addItem(std::unique_ptr<WLayoutItem> item) override;
   virtual std::unique_ptr<WLayoutItem> removeItem(WLayoutItem *item) override;
   virtual WLayoutItem *itemAt(int index) const override;
@@ -185,6 +187,11 @@ public:
     addWidget(std::unique_ptr<WWidget>(std::move(widget)), stretch, alignment);
     return result;
   }
+#else // WT_TARGET_JAVA
+  template <typename Widget>
+    Widget *addWidget(std::unique_ptr<Widget> widget);
+  template <typename Widget>
+    Widget *addWidget(std::unique_ptr<Widget> widget, int stretch);
 #endif // WT_TARGET_JAVA
 
   /*! \brief Adds a nested layout to the layout.
